#!/usr/bin/python3
import rospy
from functools import partial
import mysql.connector
import datetime
import json
from std_msgs.msg import String
from sensor_msgs.msg import NavSatFix

def shutdown_handler(args):
    db_conn = args[0]
    db_cursor = args[1]
    db_conn.close()
    db_cursor.close()
    rospy.loginfo("shuttin' down, bye")
    print("shuttin' down, bye")

def callback(data, args):
  mydb = args[0]
  mycursor = args[1]
  id_num = args[2]
  table = args[3]

  #rospy.loginfo(rospy.get_caller_id() + "%s", data)
  #rospy.loginfo("connection status: " + str(mydb.is_connected()))
  if data.header.seq % 30 == 0: # only update every 10th message to avoid over crowding the sql
    gps_data = {
      "gps" : [
        {
          "stamp" : data.header.seq,
          "timestamp" : data.header.stamp.secs,
          "lat" : data.latitude,
          "long" : data.longitude
        },
      ]
    }
    sql_data = {'deviceid' : 'drone_0',
          'taskid':'search_0',
          'gpsdata': json.dumps(gps_data),
          'created_at':datetime.datetime.now(),
          'updated_at':datetime.datetime.now()}

    sql_update = 'UPDATE ' + table + \
    ' SET {}'.format(', '.join('{}=%s'.format(k) for k in sql_data)) +\
    ' WHERE deviceid = %s'
    result = mycursor.execute(sql_update, list(sql_data.values())+['drone_0'])
    result = mydb.commit()
    rospy.loginfo("GUI gps position updated, lat: {}, lon: {}".format(data.latitude, data.longitude))

def listener():

    # In ROS, nodes are uniquely named. If two nodes with the same
    # name are launched, the previous one is kicked off. The
    # anonymous=True flag means that rospy will choose a unique
    # name for our 'listener' node so that multiple listeners can
    # run simultaneously.

    table = 'app3_gpsdata'

    mydb = mysql.connector.connect(
      host="192.168.1.100",
      #user="larkin_h",
      #passwd="Meepp973",
      database="/home/llh/sarwebui/db.sqlite3",
      #auth_plugin="mysql_native_password"
    )

    mycursor = mydb.cursor()

    id_num = 4 # id which we want to update

    rospy.init_node('db_uploader', anonymous=True)
    rospy.Subscriber("dji_osdk_ros/gps_position", NavSatFix, callback, (mydb, mycursor, id_num, table))

    # upload waypoints to GUI
    waypoint_dict = rospy.get_param("/robot_list/robot_0")
    waypoint_json = []
    for i,pt in enumerate(waypoint_dict.keys()):
        waypoint_json.append({'stamp' : waypoint_dict[pt][0], 'timestamp' : -1, 'lat' : waypoint_dict[pt][2], 'long' : waypoint_dict[pt][3]}) # leaving out altitude for now!
    
    waypoint_json = sorted(waypoint_json, key = lambda i: i['stamp'])
    wp_table = 'app3_waypointsdata'
    wp_data = {'deviceid' : 'drone_0',
               'taskid' : 'search_0',
               'waypointsdata' : json.dumps(waypoint_json),
               'created_at':datetime.datetime.now(),
               'updated_at':datetime.datetime.now()}

    sql_waypoint_update = 'UPDATE ' + wp_table + ' SET {}'.format(', '.join('{}=%s'.format(k) for k in wp_data)) + ' WHERE deviceid = %s'
    mycursor.execute(sql_waypoint_update, list(wp_data.values()) + ['drone_0'])
    mydb.commit()
    rospy.loginfo("waypoints updated")

    rospy.on_shutdown(partial(shutdown_handler, (mydb, mycursor)))
    # spin() simply keeps python from exiting until this node is stopped
    rospy.spin()

if __name__ == '__main__':
    listener()

